#!/usr/bin/env python
import xml.dom.minidom
from xml.dom.minidom import parse
from xml import xpath
from xml.dom.ext import PrettyPrint
import sys
import os.path
#sys.path.append("../../../org.glite.deployment.config/src/scripts")
from gLiteInstallerLib import UltraPrettyPrint
from  gLiteInstallerLib import gLib


def migrate_config():
    config = parse(sys.argv[1])    
    instanceNode = xpath.Evaluate('//instance[not (@service)]', config)
    if len(instanceNode) > 1:
            # there can be only one FTS instance
            print "problem ...."
            return 1     
    
    # take parameters node
    paramNode = xpath.Evaluate('/config/parameters', config)
    if len(paramNode) > 1:
            # there can be only one parameters section
            print "problem ...."
            return 1         
            
    configParams = xpath.Evaluate('//instance[not (@service)]/parameters/*', config)
    ref = paramNode[0].firstChild
    for param in configParams:
        print "Moving instance parameter: %s" % param.nodeName
        param.parentNode.removeChild(param)
        paramNode[0].insertBefore(param, ref)
        
    # remove FTS instance
    instanceNode[0].parentNode.removeChild(instanceNode[0])
    
    # All parameters from FTS instance should be moved in the central parameters section    
    # Fix the differences
    
    # Add DBBAckend
    if len(xpath.Evaluate('//file-transfer.DBBackend', config)) > 0:
        print "file-transfer.DBBackend already exists"
    else:
        new = config.createElement('file-transfer.DBBackend')
        new.setAttributeNS('','value', 'oracle')
        paramNode[0].appendChild(new) 
    
    # replace '${file-transfer.VONAME}/' by '' in all nodes
    print "Removing ${file-transfer.VONAME} references"
    allNodes = xpath.Evaluate('//*', config)
    for node in allNodes:
        value = node.getAttribute('value')
        if value.find('${file-transfer.VONAME}') > -1:
            value = value.replace('${file-transfer.VONAME}/','')
            value = value.replace('${file-transfer.VONAME}-','')
            node.setAttribute('value', value)
            print "Fixed %s; new value %s" %(node.nodeName, value)
                    
    vonameNode = xpath.Evaluate('//file-transfer.VONAME', config)
    for node in vonameNode:
        node.parentNode.removeChild(node)
        
    config_file = os.path.basename(sys.argv[1])
    file = open('%s.generated' % config_file, "w")
    #PrettyPrint(config.documentElement)
    UltraPrettyPrint(config.documentElement, stream=file)
    file.close()        
    return 0   
    
    
if len(sys.argv) != 2:
    print "Usage:\nfts-config-migrator-1.3-1.4.py <1.3 config_file>"
    sys.exit(1)
    
glib = gLib()
required_version = '2.1.0'    
# stop the service
print("Stoping tomcat                              ")
if os.system('/sbin/service tomcat5 stop') != 0:
    print "Cannot stop service"
    glib.printFailedMessage()
    sys.exit(1)
if os.path.exists('/etc/cron.d/watchdog.cron'):
    os.remove('/etc/cron.d/watchdog.cron')
glib.printOkMessage()

# remove the web application
print("Removing tomcat cache directories           ")
os.system('rm -rf /usr/share/tomcat5/work/Catalina/localhost/glite-data-transfer-fts')
os.system('rm -rf /usr/share/tomcat5/webapps/glite-data-transfer-fts')
glib.printOkMessage()

# migrate config
print("Migrating configuration file                ")
if migrate_config() != 0:
    glib.printFailedMessage()
    glib.printErrorMessage("Error during configuration file migration")
    sys.exit(1)
glib.printOkMessage()


config_file = os.path.basename(sys.argv[1])
# backup old config
os.system("cp %s %s.1.3" %(config_file, config_file))
# rename generated config
os.system("mv %s.generated glite-file-transfer-service.cfg.xml" % config_file) 

# Load parameters
params = {}

print("Upgrade DB schema                            ")
# upgrade the DB schemas
if glib.loadConfiguration("%s/../glite-file-transfer-service.cfg.xml" % glib.getScriptPath(), params):
    print "An error occurred while configuring the service"
    sys.exit(1)



################################
#    Check schemas
################################
#Format: username/password@host:port/servicename
os.environ["LD_LIBRARY_PATH"]= "%s/lib/" % params['file-transfer.oracle-instantclient.location']

exist = 0
if params['file-transfer.DBBackend'] == 'oracle':
    sqlplusConnectString = "%s/%s@%s:%s/%s" %(params['file-transfer.DBUSER'],\
        params['file-transfer.DBPASSWORD'],params['file-transfer.DBHOST'],\
        params['file-transfer.DBPORT'], params['file-transfer.DBSERVICENAME']) 
         
    command = "sqlplus -S %s @%s/etc/glite-data-transfer-fts/schema/oracle/oracle-get-version.sql"\
        %(sqlplusConnectString, params['GLITE_LOCATION'])
    file = os.popen(command, 'r')
    version = file.read()
    if file.close() is None:
        exist = 1
else:
    if os.system("mysql -u%s -p%s %s -e'exit' > /dev/null 2>&1" \
        %(params['file-transfer.DBUSER'], params['file-transfer.DBPASSWORD'], params['file-transfer.DBSERVICENAME'])) != 0:
            glib.printErrorMessage("Error during communication with the database")
            sys.exit(1)
    file = os.popen("mysql -u%s -p%s %s < %s/etc/glite-data-transfer-fts/schema/mysql/mysql-get-version.sql 2>&1" \
        %(params['file-transfer.DBUSER'], params['file-transfer.DBPASSWORD'], params['file-transfer.DBSERVICENAME'],\
        params['GLITE_LOCATION']), 'r')
    ret = file.readlines()
    file.close()                
    if len(ret) == 2:
        version = ret[1]
        exist = 1

if exist == 1:
    print "Schema version loaded in the database is: %s" % version
    if version > required_version:
        print "No schema upgrade needed"
        sys.exit(0)
    else:
        print "Schema is going to be upgraded"
else:
    print "No schema found in the database, no migration needed"
    sys.exit(0)

################################
#    Load schemas
################################
# /opt/glite/etc/glite-data-transfer-fts/schema/oracle/oracle-upgrade_2.0.0-2.1.0.sql
print "Upload database schema                       "
if params['file-transfer.DBBackend'] == 'oracle':        
    # the common transfer schema first
    command = "sqlplus %s @%s/etc/glite-data-transfer-fts/schema/oracle/oracle-upgrade_2.0.0-2.1.0.sql" \
        %(sqlplusConnectString , os.environ['GLITE_LOCATION'])
    print "Command1: %s" %command
    file = os.popen(command,'w',0)
    file.write('quit')
    file.close()
    # TBD check for return value
else:
    print "No MySQL schema upgrade script provided"
    sys.exit(1)
glib.printOkMessage()

