#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-file-transfer-service-oracle-config v. 3.2.0
#
# Post-installation script for configuring the gLite data Local Transfer Service
# Robert Harakaly  < robert.harakaly@cern.ch >
#
# Version info: $Id: glite-file-transfer-service-config.py,v 1.7 2006/03/02 10:08:25 flammer Exp $
#
# Usage: python glite-file-transfer-service-config [OPTION]
#        -c                                      print configuration
#        -v                                      print version
#        -h,--help                               print usage info
#        --configure                            configure the service
#        --remove                               remove the database schema and data
#        --start                                start the service
#        --stop                                 start the service
#        --status                               show status of service
#        --siteconfig=<site config URL>         use remote site configuration file from URL
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################

import os,string,socket,pwd,time
import sys
import getopt
import commands

from gLiteInstallerLib import gLib
from gLiteInstallerLib import ConfigParams
from tomcat5 import Secure_tomcat5 as Tomcat5
from gliteRgmaServicetool import gliteRgmaServicetoolInstance
from gliteRgmaServicetool import gliteRgmaServicetool
from serviceDiscovery import ServiceDiscovery

# Set global variables here 
global params                         # all config values from the XML file
global verbose
global glib
global tomcat                        # the one and only tomcat service instance

class glite_data_local_transfer_service:

    def __init__(self,params):
        self.params = params
        self.verbose = 0
        self.version = "3.2.0"
        self.name = "glite-file-transfer-service"
        self.friendly_name = "gLite File Transfer Service"

        self.transferFts = glite_data_transfer_fts(self.params)
        
    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    #-------------------------------------------------------------------------------
    # Usage
    #-------------------------------------------------------------------------------

    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION]""" % (self.name, os.environ['GLITE_LOCATION'], \
        os.environ['GLITE_LOCATION'], self.name, self.name)

        print '    -c, --checkconf                       print the service configuration'
        print '    -v, --version                         print the version of the configuration script'
        print '    -h, --help                            print this usage information'
        print '    --configure                           configure the service and database (default)'
        print '    --remove                              remove the service and database schemas'
        print '    --start                               start the service'
        print '    --stop                                stop the service'
        print '    --status                              status of service'
        print '    --siteconfig=<site config url>        use site configuration file'
        print '\n'


    #################################################################
    # configure
    #
    # configure file transfer service client
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def configure(self):    
        if os.path.exists("%s/etc/services.xml" % os.environ['GLITE_LOCATION']):
            glib.printInfoMessage("Creating backup of the services.xml file")
            os.system("cp %s/etc/services.xml %s/etc/services.xml.back" \
                % (os.environ['GLITE_LOCATION'], os.environ['GLITE_LOCATION']))
        
        if params['file-transfer.DBBackend'] == 'oracle':
            #-------------------------------------------------------------------------------
            # Check Oracle instanclient libs
            #-------------------------------------------------------------------------------
            if params.has_key('file-transfer.oracle-instantclient.location'):
                if params['file-transfer.oracle-instantclient.location'] != "":
                    if not os.path.exists(params['file-transfer.oracle-instantclient.location']):
                        glib.printErrorMessage("The Oracle InstantClient cannot be found at the specified location")
                        glib.printErrorMessage(params['file-transfer.oracle-instantclient.location'])
                        glib.printErrorMessage("Configuration cannot continue")
                        return 1
                    
            ##################################
            #    Copy Oracle jar file in tomcat location
            ##################################
            os.system("cp %s/lib/ojdbc14.jar %s/common/lib/" % (params['file-transfer.oracle-instantclient.location'], os.environ["CATALINA_HOME"]))    
            
            #-------------------------------------------------------------------
            # Check for Oracle JDBC drivers
            #-------------------------------------------------------------------
            if not os.path.exists('%s/common/lib/ojdbc14.jar' % os.environ["CATALINA_HOME"]):
                glib.printErrorMessage("Oracle drivers cannot be found. Cannot continue. %s" % os.environ["CATALINA_HOME"])
                glib.printErrorMessage("Please install the Oracle JDBC drivers into the  directory %s/common/lib/" % os.environ["CATALINA_HOME"])
                glib.printErrorMessage("and restart the configuration script")
                return 1    

        #-------------------------------------------------------------------------------
        # Verify the JAVA installation and set it in /etc/java.conf
        #-------------------------------------------------------------------------------
        
        if glib.checkJava() != 0:
            return 1
        
        # ------------------------------------------------------------
        # configure service discovery
        # ------------------------------------------------------------
        print "\nConfiguring Service Discovery ...                       "
        errorcode =    serviceDiscovery.configure(params,glib)
        if errorcode:
            glib.printFailedMessage()
            return 1
        glib.printOkMessage()

        #-------------------------------------------------------------------------------
        # Configure Tomcat
        #-------------------------------------------------------------------------------
        
        print "\nConfiguring Tomcat ...                                  "
        
        # stop tomcat first
        tomcat.stop()
        
        # configure tomcat
        errorcode = tomcat.configure()
        if errorcode != 0:
            print "Configuring Tomcat                                             ",
            glib.printFailedMessage()
            return 1
             
        print "Configuring Tomcat                                           ",
        glib.printOkMessage()

        
        #--------------------------------------------------------
        # Configure the Security Utilities
        #--------------------------------------------------------        
        if os.system("python %s/glite-security-utils-config.py --subservice" % glib.getScriptPath()):
            print "\nConfiguring gLite Security Utilities                      ",
            glib.printFailedMessage()
            return 1

        print "\nConfiguring gLite Security Utilities                      ",
        glib.printOkMessage()

        #--------------------------------------------------------
        # Call config of gliteDataTransferFts
        #--------------------------------------------------------
        if self.transferFts.configure() != 0:
            return 1
    
        #-------------------------------------------------------------------
        # RGMA servicetool: configure servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
            
            # Instantiate the rgma-servicetool class
            rgmaServicetool = gliteRgmaServicetool()
            rgmaServicetool.verbose = self.verbose
            
            # Create FTS instance
            serviceId = "%s_%s" % (glib.fq_hostname, params['fts.serviceType'])
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
            
            # set params
            servicetoolInstance.setServiceName(params['fts.serviceName'])
            servicetoolInstance.setServiceType(params['fts.serviceType'])
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript(params['fts.statusScript'])
            servicetoolInstance.setEndpoint(params['fts.endpoint'])
            servicetoolInstance.setWsdlUrl(params['fts.wsdlurl'])
            
            # add instance to the gLite configuration
            if servicetoolInstance.add() == 1:
                return 1
            
            # Create ChannelManagement instance
            serviceId = "%s_%s" % (glib.fq_hostname, params['channelManagement.serviceType'])
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
            
            # set params
            servicetoolInstance.setServiceName(params['channelManagement.serviceName'])
            servicetoolInstance.setServiceType(params['channelManagement.serviceType'])
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript(params['channelManagement.statusScript'])
            servicetoolInstance.setEndpoint(params['channelManagement.endpoint'])
            servicetoolInstance.setWsdlUrl(params['channelManagement.wsdlurl'])
            
            # add instance to the gLite configuration
            if servicetoolInstance.add() == 1:
                return 1
            
            # Create ileTransferStats instance
            serviceId = "%s_%s" % (glib.fq_hostname, params['fileTransferStats.serviceType'])
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
            
            # set params
            servicetoolInstance.setServiceName(params['fileTransferStats.serviceName'])
            servicetoolInstance.setServiceType(params['fileTransferStats.serviceType'])
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript(params['fileTransferStats.statusScript'])
            servicetoolInstance.setEndpoint(params['fileTransferStats.endpoint'])
            servicetoolInstance.setWsdlUrl(params['fileTransferStats.wsdlurl'])
            
            # add instance to the gLite configuration
            if servicetoolInstance.add() == 1:
                return 1
            
            # Configure servicetool
            if rgmaServicetool.configure(glib):
                # error in configuring servicetool
                return 1
            
        return 0


    def remove(self):

        if self.transferFts.remove() != 0:
            return 1
        return 0
        
    def start(self):
        if not os.path.exists("%s/conf/Catalina/localhost/glite-data-transfer-fts.xml" \
            % os.environ["CATALINA_HOME"]):
                glib.printErrorMessage("Tomcat context file doesn't exist. Verify that configuration is succesfully finished")
                return 1
                
        if params['file-transfer.DBBackend'] == 'oracle':
            #-------------------------------------------------------------------------------
            # Check Oracle instanclient libs
            #-------------------------------------------------------------------------------
            if params.has_key('file-transfer.oracle-instantclient.location'):
                if params['file-transfer.oracle-instantclient.location'] != "":
                    if not os.path.exists(params['file-transfer.oracle-instantclient.location']):
                        glib.printErrorMessage("The Oracle InstantClient cannot be found at the specified location")
                        glib.printErrorMessage(params['file-transfer.oracle-instantclient.location'])
                        glib.printErrorMessage("Cannot start the %s" % self.friendly_name)
                        return 1
        else:
            # Check if MySQL DB is running
            pass
            
        if tomcat.start() != 0:
            return 1
        
        #-------------------------------------------------------------------
        # Start Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
        
            errorcode = rgmaServicetool.start()
            if (errorcode != 0):
                return 1
        
        # Tomcat
        print "Waiting until the tomcat server fully starts                  "
        time.sleep(10)
        glib.printOkMessage()
        
        # Watchdog
        if params["watchdog.enable"] == 'true':
            print "Enabling watchdog                                          "
            self.addWatchdog()     
            glib.printOkMessage()
            
        return 0
        
    def stop(self):
        
        error_level = 0
        
        if params["watchdog.enable"] == 'true':
            self.stopWatchdog()        

        if tomcat.stop() != 0:
            error_level = 1

        #-------------------------------------------------------------------
        # Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
            
            if rgmaServicetool.stop():
                error_level = 1
    
        return error_level


    #################################################################
    # status
    #
    # print status of FPS
    #
    # return 0 ok
    #        1 error
    #
    #################################################################
    def status(self):
        
        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "       Checking status of %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        errorcode = 0
        
        if params.has_key('host.certificate.file'):
            hostcert_file = params['host.certificate.file']
        else:
            hostcert_file = "/etc/grid-security/hostcert.pem"
        
        if params.has_key('host.key.file'):    
            hostkey_file  = params['host.key.file']
        else:
            hostkey_file = "/etc/grid-security/hostkey.pem"
        
        status = glib.checkHTTPSconnection("localhost", 
            "/glite-data-transfer-fts/services/FileTransfer?method=getVersion",
            8443, hostkey_file, hostcert_file)
                                    
        print "FTS is ",
        if status[0]:
            print "not running."
            errorcode = 1
        else:
            print "running."
                
        status = glib.checkHTTPSconnection("localhost", 
             "/glite-data-transfer-fts/services/ChannelManagement?method=getVersion",
            8443, hostkey_file, hostcert_file)
                                    
        print "ChannelManagement is ",
        if status[0]:
            print "not running."
            errorcode = 1
        else:
            print "running."

        #-------------------------------------------------------------------
        # Servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
            
            if rgmaServicetool.status() != 0:
                errorcode = 1
    
        return errorcode

    def addWatchdog(self):

        print "Adding watchdog configuration"
        glib.addWatchdog('transfer-fts', params['watchdog.fts.check-command'], params["watchdog.fts.return-string"],\
                        '/etc/init.d/tomcat5 restart',"")
        return 0
        
    def stopWatchdog(self):
        print "Removing watchdog configuration"
    
        watchDog_xml = "%s/etc/watchdog.d/transfer-fts.xml" % (params['GLITE_LOCATION'])
        if os.path.exists(watchDog_xml):
            os.remove(watchDog_xml)
        return 0
        
class glite_data_transfer_fts:

    def __init__(self, params):
        pass
            
    def configure(self): 

        print "Configure transfer-fts"
        
        glib.print_params(params)

        ##########################################################
        # Checking VETO mapfile
        ##########################################################
        if not os.path.exists('%s' %params['file-transfer.VETO_MAPFILE']):
            os.system("touch %s" %params['file-transfer.VETO_MAPFILE']);

        ##########################################
        #    create default mapfiles to enable the watchdog functionality
        ##########################################
        for key in params.keys():
            if key.find("MAPFILE") > -1 :
                if params[key] != "" :
                    mapfile = params[key]
                    if not os.path.exists(mapfile):
                        f = open(mapfile,"w")
                        f.write('"%s" null' % glib.get_certificate_subject(params["host.certificate.file"],"/"))
                        f.close()

        ################################
        #    Deploy needed jar files in tomcat
        ################################
        os.system("cp %s/lib/glite-data-common/* /var/lib/tomcat5/common/lib/" %os.environ['GLITE_LOCATION'])
                        
        ################################
        #    Check schemas
        ################################
        #Format: username/password@host:port/servicename
        exist = 0
        if params['file-transfer.DBBackend'] == 'oracle':
            sqlplusConnectString = "%s/%s@%s:%s/%s" %(params['file-transfer.DBUSER'],\
                params['file-transfer.DBPASSWORD'],params['file-transfer.DBHOST'],\
                params['file-transfer.DBPORT'], params['file-transfer.DBSERVICENAME']) 
            
            command = "sqlplus -S %s @%s/etc/glite-data-transfer-fts/schema/oracle/oracle-get-version.sql"\
                %(sqlplusConnectString, os.environ['GLITE_LOCATION'])
            file = os.popen(command, 'r')
            version = file.read()
            if file.close() is None:
                exist = 1
        else:
            if os.system("mysql -h%s -u%s -p%s %s -e'exit' > /dev/null 2>&1" \
                %(params['file-transfer.DBHOST'],params['file-transfer.DBUSER'], params['file-transfer.DBPASSWORD'], params['file-transfer.DBSERVICENAME'])) != 0:
                    glib.printErrorMessage("Error during communication with the database.\nVerify if:\n- MySQL client is installed\
                        \n- MySQL database is running\n- database is configured with correct access privilegies\n")
                    return 1
            file = os.popen("mysql -h%s -u%s -p%s %s < %s/etc/glite-data-transfer-fts/schema/mysql/mysql-get-version.sql 2>&1" \
                %(params['file-transfer.DBHOST'],params['file-transfer.DBUSER'], params['file-transfer.DBPASSWORD'], params['file-transfer.DBSERVICENAME'],\
                os.environ['GLITE_LOCATION']), 'r')
            ret = file.readlines()
            file.close()                
            if len(ret) == 2:
                version = ret[1]
                exist = 1
        
        ################################
        #    Replace tokens in the context.xml template file
        ################################

        inFile = open("%s/etc/glite-data-transfer-fts/context.xml.template" \
            % os.environ['GLITE_LOCATION'],"r")
        outFile = open("%s/conf/Catalina/localhost/glite-data-transfer-fts.xml" \
            % os.environ["CATALINA_HOME"] ,"w")
        for line in inFile.readlines():
            line = line.replace("@PATH@", "/glite-data-transfer-fts")
            line = replaceTags(line, params, 'file-transfer-fts')
            line = replaceTags(line, params, 'file-transfer')
            line = replaceTags(line, params, '', 1)
            outFile.write(line)
        outFile.close()
        inFile.close()
                 
        if exist == 1:
            print "There is already a schema loaded into the database, with version: %s" % version
            return 0
            # check if schema exist and if yes exit .....
        
        ################################
        #    Load schemas
        ################################
        print "Upload database schema                       "
        if params['file-transfer.DBBackend'] == 'oracle':        
            # the common transfer schema first
            command = "sqlplus %s @%s/etc/glite-data-transfer-fts/schema/oracle/oracle-schema.sql" \
                %(sqlplusConnectString , os.environ['GLITE_LOCATION'])
            print "Command1: %s" %command
            file = os.popen(command,'w',0)
            file.write('quit')
            file.close()
            # TBD check for return value
        else:
            if os.system("mysql -h%s -u%s -p%s %s < %s/etc/glite-data-transfer-fts/schema/mysql/mysql-schema.sql" \
                %(params['file-transfer.DBHOST'],params['file-transfer.DBUSER'], params['file-transfer.DBPASSWORD'], params['file-transfer.DBSERVICENAME'],\
                os.environ['GLITE_LOCATION'])) == 1:
                    glib.printFailedMessage()
                    return 1
                    
        glib.printOkMessage()
        return 0
                        
    def remove(self):
        print "Removing transfer-fts"

        ################################
        #    Remove context.xml file
        ################################
        context = "%s/conf/Catalina/localhost/glite-data-transfer-fts.xml" % (os.environ["CATALINA_HOME"])
        print "Removing context: %s" % context
        if os.path.exists(context):
            os.remove(context)

        ################################
        #    Check schema
        ################################
        #Format: username/password@host:port/servicename
        print "Checking if DB schema is already uploaded             "
        if params['file-transfer.DBBackend'] == 'oracle':        
            sqlplusConnectString = "%s/%s@%s:%s/%s" %(params['file-transfer.DBUSER'],\
                params['file-transfer.DBPASSWORD'],params['file-transfer.DBHOST'],\
                params['file-transfer.DBPORT'], params['file-transfer.DBSERVICENAME']) 
            
            command = "sqlplus -S %s @%s/etc/glite-data-transfer-fts/schema/oracle/oracle-get-version.sql"\
                %(sqlplusConnectString, os.environ['GLITE_LOCATION'])
            file = os.popen(command, 'r')
            version = file.read()
            if not file.close() is None:
                print "No schema found: nothing to do!"
                return 0
        else:
            file = os.popen("mysql -h%s -u%s -p%s %s < %s/etc/glite-data-transfer-fts/schema/mysql/mysql-get-version.sql 2>&1" \
                %(params['file-transfer.DBHOST'],params['file-transfer.DBUSER'], params['file-transfer.DBPASSWORD'], params['file-transfer.DBSERVICENAME'],\
                os.environ['GLITE_LOCATION']), 'r')
            ret = file.readlines()
            file.close()
            if len(ret) != 2:
                print "No schema found: nothing to do!"
                return 0
                        
        ################################
        #    Remove schema
        ################################
        print "Removing database schema                "
        if params['file-transfer.DBBackend'] == 'oracle':        
            command = "sqlplus %s @%s/etc/glite-data-transfer-fts/schema/oracle/oracle-drop.sql"\
                %(sqlplusConnectString, os.environ['GLITE_LOCATION'])
            print "Command2: %s" %command
            file = os.popen(command,'w',0)
            file.write('quit')
            file.close()
            #check for return value
        else:
            if os.system("mysql -h%s -u%s -p%s %s < %s/etc/glite-data-transfer-fts/schema/mysql/mysql-drop.sql" \
                %(params['file-transfer.DBHOST'],params['file-transfer.DBUSER'], params['file-transfer.DBPASSWORD'], params['file-transfer.DBSERVICENAME'],\
                os.environ['GLITE_LOCATION'])) == 1:
                    glib.printFailedMessage()
                    return 1
                    
        glib.printOkMessage()    
        return 0

def replaceTags(line, params, prefix,reset=0):
    if line.find("@") > -1:
        tags = line.split("@")
        i = 1
        while i < len(tags)-1:
            if params.has_key(prefix + "." + tags[i]):
                paramName = "%s.%s" % (prefix, tags[i])
                print("Replace @%s@ by %s" % (tags[i], params[paramName]))
                line = line.replace("@%s@" % tags[i], params[paramName])
            else:
                if reset:
                    print("Replace @%s@ by empty string" %(tags[i]))
                    line = line.replace("@%s@" % tags[i], "")
            i += 2
    return line
        
#-------------------------------------------------------------------------------
# Default param values
#-------------------------------------------------------------------------------

def loadDefaults(params):

    params['GLITE_LOCATION'] = "/opt/glite"
    
    params['fts.serviceName'] = 'File Transfer Service at %s' % glib.fq_hostname
    params['fts.serviceType'] = 'org.glite.FileTransfer'
    params['fts.statusScript'] = 'wget -S -r https://%s:8443/glite-data-transfer-fts/services/FileTransfer' % glib.fq_hostname
    params['fts.endpoint'] = 'https://%s:8443/glite-data-transfer-fts/services/FileTransfer' % glib.fq_hostname
    params['fts.wsdlurl'] = 'https://%s:8443/glite-data-transfer-fts/services/FileTransfer?wsdl'  % glib.fq_hostname
    
    params['channelManagement.serviceName'] = 'Channel Management service at %s' % glib.fq_hostname
    params['channelManagement.serviceType'] = 'org.glite.ChannelManagement'
    params['channelManagement.statusScript'] = 'wget -S -r https://%s:8443/glite-data-transfer-fts/services/ChannelManagement' % glib.fq_hostname
    params['channelManagement.endpoint'] = 'https://%s:8443/glite-data-transfer-fts/services/ChannelManagement' % glib.fq_hostname
    params['channelManagement.wsdlurl'] = 'https://%s:8443/glite-data-transfer-fts/services/ChannelManagement?wsdl'  % glib.fq_hostname
    
    params['fileTransferStats.serviceName'] = 'File Trasfer Statistics service at %s' % glib.fq_hostname
    params['fileTransferStats.serviceType'] = 'org.glite.FileTransferStats'
    params['fileTransferStats.statusScript'] = 'wget -S -r https://%s:8443/glite-data-transfer-fts/services/FileTransferStats' % glib.fq_hostname
    params['fileTransferStats.endpoint'] = 'https://%s:8443/glite-data-transfer-fts/services/FileTransferStats' % glib.fq_hostname
    params['fileTransferStats.wsdlurl'] = 'https://%s:8443/glite-data-transfer-fts/services/FileTransferStats?wsdl'  % glib.fq_hostname
    
#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def set_env():
    glib.export('GLITE_LOCATION')
    glib.export('GLITE_LOCATION_VAR')
    glib.export('GLITE_LOCATION_LOG')
    glib.export('GLITE_LOCATION_TMP')
    glib.export('GLOBUS_LOCATION')
    glib.export('GPT_LOCATION')
    glib.export('LD_LIBRARY_PATH')
    
    glib.check_dir(os.environ['GLITE_LOCATION_VAR'])
    glib.check_dir(os.environ['GLITE_LOCATION_LOG'])
    glib.check_dir(os.environ['GLITE_LOCATION_TMP'])
    
    glib.export("JAVA_HOME")
    glib.export("CATALINA_HOME","/var/lib/tomcat5")

    glib.addEnvPath("PATH", "%s/bin" % os.environ["JAVA_HOME"])
    glib.addEnvPath("PATH", "%s/externals/bin" % os.environ["GLITE_LOCATION"])
    glib.addEnvPath("PATH", "%s/bin" % os.environ["GLITE_LOCATION"])

    glib.addEnvPath("LD_LIBRARY_PATH", "/usr/local/lib")
    glib.addEnvPath("LD_LIBRARY_PATH", "%s/lib" % os.environ['GLOBUS_LOCATION'])
    glib.addEnvPath("LD_LIBRARY_PATH", "%s/externals/lib" % os.environ["GLITE_LOCATION"])
    glib.addEnvPath("LD_LIBRARY_PATH", "%s/lib" % os.environ["GLITE_LOCATION"])
    if params['file-transfer.DBBackend'] == 'oracle':
        glib.addEnvPath("LD_LIBRARY_PATH", "%s/lib/" % params['file-transfer.oracle-instantclient.location'])
    
    # Perl
    glib.addEnvPath("PERL5LIB", "%s/lib/perl:%s/lib/perl5" % (os.environ['GPT_LOCATION'],os.environ['GLITE_LOCATION']))

    # Fix X509_USER_{CERT|KEY} problem in setenv file
    if os.path.exists(params['installer.export.filename']):
        infile = open(params['installer.export.filename'],'r')
        outfile = open(params['installer.export.filename']+'.1','w')
        for line in infile:
            if line.find('X509_USER_CERT') == -1 and \
                line.find('X509_USER_KEY') == -1:
                outfile.write(line)
                
        outfile.close()
        infile.close()
        os.rename(params['installer.export.filename']+'.1',params['installer.export.filename'])

def setDefaults():
    if params['file-transfer.DBBackend'] == 'oracle':
        if params['file-transfer.DBPORT'] == '':
            params['file-transfer.DBPORT'] = '1521'
        if params['file-transfer.DBURL'] == '':
            #params['file-transfer.DBURL'] = "jdbc:oracle:thin:@(DESCRIPTION=(ADDRESS=(PROTOCOL=TCP)(HOST=${file-transfer.DBHOST})(PORT=${file-transfer.DBPORT}))(CONNECT_DATA=(SERVICE_NAME=${file-transfer.DBSERVICENAME})))"
            params['file-transfer.DBURL'] = "jdbc:oracle:thin:@(DESCRIPTION=(ADDRESS=(PROTOCOL=TCP)(HOST=%s)(PORT=%s))(CONNECT_DATA=(SERVICE_NAME=%s)))"\
                %(params['file-transfer.DBHOST'], params['file-transfer.DBPORT'],params['file-transfer.DBSERVICENAME'])
        if params['file-transfer-fts.DBDRIVERCLASS'] == '':
            params['file-transfer-fts.DBDRIVERCLASS'] = "oracle.jdbc.driver.OracleDriver"
    else:
        if params['file-transfer.DBPORT'] == '':
            params['file-transfer.DBPORT'] = '3306'
        if params['file-transfer.DBURL'] == '':
            params['file-transfer.DBURL'] = "jdbc:mysql://%s:%s/%s" \
                %(params['file-transfer.DBHOST'], params['file-transfer.DBPORT'],params['file-transfer.DBSERVICENAME'])
        if params['file-transfer-fts.DBDRIVERCLASS'] == '':
            params['file-transfer-fts.DBDRIVERCLASS'] = "org.gjt.mm.mysql.Driver"

#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid()==0:
        print '"\nThis script must be run as root\n'
        sys.exit(1)
        
    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # Get an instance of the library class
    glib = gLib()
    glib.verbose = 0
    
    # Load parameters
    loadDefaults(params)
    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass
    if glib.loadConfiguration(["%s/../glite-file-transfer-service.cfg.xml" % glib.getScriptPath(),\
            "%s/../glite-service-discovery.cfg.xml" % glib.getScriptPath(),
            "%s/../glite-rgma-servicetool.cfg.xml" % glib.getScriptPath()],params):
        print "An error occurred while configuring the service"
        sys.exit(1)

    setDefaults()
    
    verbose = 0
    if params.has_key('glite.installer.verbose'):
        if params['glite.installer.verbose'] == "true":
            verbose = 1
    gLib.verbose = verbose
    
    # Set up the environment
    set_env()
          
    # Instantiate the service classes
    service = glite_data_local_transfer_service(params)
    service.verbose = verbose
 
    # Instantiate the tomcat class
    tomcat = Tomcat5(params)
    tomcat.verbose = verbose

    # Instantiate the servicediscovery class
    serviceDiscovery = ServiceDiscovery()
    serviceDiscovery.verbose = verbose
    
    # Instantiate the rgma servicetool class
    if not params.has_key('rgma.servicetool.activate'):
        params['rgma.servicetool.activate'] = 'true'
    if params['rgma.servicetool.activate'] == 'true':
        rgmaServicetool = gliteRgmaServicetool()
        rgmaServicetool.verbose = verbose

    # Command line opts if any
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', 'configure', 'start', 'stop', 'status', 'remove', 'siteconfig='])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    if len(opts) == 0:
        service.usage()
        sys.exit(0)
    
    # Check cli options
    for o, a in opts:

        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)

        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)

        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)

        if o == "--configure":

            # Check certificates
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if glib.check_certs(params) != 0:
                        print "An error occurred while configuring the %s" \
                            % service.friendly_name
                        sys.exit(1)
            
            service.copyright()
            service.showVersion()
            service.banner()
                
            # Display the parameters if verbose output
            if verbose:
                glib.print_params(params)

                
            # Stop all services
            glib.printInfoMessage("\n\nStopping all running FTS services...")
            service.stop()
            
            # Configure the service
            return_value = service.configure()
            
            if return_value == 0:

                # Stop all services
                glib.printInfoMessage("\n\nStopping all running FTS services...")
                service.stop()
                
                glib.registerService()

                print "\n\nThe %s configuration was successfully completed\n" % service.friendly_name
                print "You can now start the service using the --start option of this script\n\n"
    
                sys.exit(0)

            elif return_value == 2:

                # Stop all services
                glib.printInfoMessage("\n\nStopping all running FTS services...")
                service.stop()
                
                print "\n\nThe %s configuration was completed, but one or more warnings" % service.friendly_name
                print "were issued. Please revise them and re-run the script"
                print "or configure and start the %s manually\n" % service.friendly_name

                sys.exit(2)

            else:
                print "\n\nAn unrecoverable error occurred while configuring the %s service" \
                    % service.friendly_name

                sys.exit(1)
            
        if o == "--start":
            if service.start() == 0:
                print "\n\nThe %s was successfully started\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while starting the %s service" \
                    % service.friendly_name
                sys.exit(1)

        if o == "--stop":
            if service.stop() == 0:
                print "\n\nThe %s was successfully stopped\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while stopping the %s service" \
                    % service.friendly_name
                sys.exit(1)

        if o == "--status":
            errorcode = service.status()
            if errorcode:
                sys.exit(1)
            sys.exit(0)
        if o == "--remove":
            glib.printInfoMessage("\n\nStopping all running FTS services...")
            if service.stop() != 0:
                sys.exit(1)

            retVal = service.remove()
            print "\n\nAll service instances and database schemas were successfully removed\n"
            sys.exit(retVal)
            if service.stop() != 0:
                sys.exit(1)
            sys.exit(service.remove())


